@@ -1,22 +1,19 @@
-#undef NAUTILUS_BUILD_MESSAGE
-#undef NAUTILUS_SOURCE_DIRECTORY
-#undef EAZEL_BUILD_TIMESTAMP
+#undef ENABLE_INSTALLER
 #undef ENABLE_NLS
 #undef ENABLE_PROFILER
-#undef ENABLE_INSTALLER
+#undef GETTEXT_PACKAGE
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
-#undef HAVE_MEDUSA
-#undef HAVE_LIBJPEG
-#undef HAVE_STPCPY
 #undef HAVE_LIBBZ2
-#undef HAVE_AMMONITE
+#undef HAVE_LIBJPEG
+#undef HAVE_MEDUSA
+#undef HAVE_STPCPY
+#undef bzclose
 #undef bzCompress
 #undef bzCompressInit
 #undef bzDecompress
 #undef bzDecompressInit
-#undef bzclose
 #undef bzdopen
 #undef bzerror
 #undef bzflush
