@@ -39,6 +39,7 @@
 #include <gtk/gtkmain.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-monitor.h>
 #include <libxml/parser.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -584,6 +585,51 @@ nautilus_directory_set_up_request (Request *request,
 
 }
 
+static void
+mime_db_changed_callback (GnomeVFSMIMEMonitor *ignore, NautilusDirectory *dir)
+{
+	const Monitor *monitor;
+	GList *ptr, *attrs;
+	GList *file_list;
+
+	g_return_if_fail (dir != NULL);
+	g_return_if_fail (dir->details != NULL);
+
+	attrs = NULL;
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES);
+
+	file_list = NULL;
+	for (ptr = dir->details->monitor_list ; ptr != NULL ; ptr = ptr->next) {
+		monitor = ptr->data;
+		if (monitor->request.file_info && monitor->file != NULL) {
+			if (nautilus_file_is_self_owned (monitor->file)) {
+				nautilus_file_emit_changed (monitor->file);
+				nautilus_file_invalidate_attributes (monitor->file, attrs);
+			} else {
+				file_list = g_list_prepend (file_list, 
+							    monitor->file);
+			}
+			
+		}
+	}
+	
+	if (file_list) {
+		nautilus_directory_emit_change_signals (dir, file_list);
+
+		for (ptr = file_list; ptr != NULL; ptr = ptr->next) {
+			nautilus_file_invalidate_attributes (ptr->data, attrs);
+		}
+		g_list_free (file_list);
+	}
+	g_list_free (attrs);
+}
+
 void
 nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 					 NautilusFile *file,
@@ -641,6 +687,13 @@ nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 		nautilus_directory_register_metadata_monitor (directory);	
 	}
 
+	if (monitor->request.file_info && directory->details->mime_db_monitor == 0) {
+		directory->details->mime_db_monitor = g_signal_connect_object (
+			gnome_vfs_mime_monitor_get (),
+			"data_changed",
+			G_CALLBACK (mime_db_changed_callback), directory, 0);			  
+	}
+
 	/* Put the monitor file or all the files on the work queue. */
 	if (file != NULL) {
 		nautilus_directory_add_file_to_work_queue (directory, file);
@@ -2478,7 +2531,7 @@ mime_list_start (NautilusDirectory *directory,
 	if (!nautilus_file_is_directory (file)) {
 		g_list_free (file->details->mime_list);
 		file->details->mime_list_failed = FALSE;
-		file->details->got_directory_count = FALSE;
+		file->details->got_mime_list = FALSE;
 		file->details->mime_list_is_up_to_date = TRUE;
 
 		nautilus_directory_async_state_changed (directory);
@@ -3182,6 +3235,8 @@ nautilus_directory_cancel_loading_file_attributes (NautilusDirectory *directory,
 {
 	Request request;
 	
+	nautilus_directory_remove_file_from_work_queue (directory, file);
+
 	nautilus_directory_set_up_request (&request,
 					   file_attributes);
 
@@ -3263,6 +3318,8 @@ void
 nautilus_directory_add_file_to_work_queue (NautilusDirectory *directory,
 					   NautilusFile *file)
 {
+	g_return_if_fail (file->details->directory == directory);
+
 	if (!file_needs_work_done (directory, file)) {
 		return;
 	}
