@@ -138,10 +138,13 @@ char **broken_cdda_interface_h_workaround = strerror_tr;
 
 #ifdef HAVE_SYS_MNTTAB_H
 typedef struct mnttab MountTableEntry;
+#define MOUNT_TABLE_ENTRY_TYPE(ent) ((ent)->mnt_fstype)
 #elif defined (HAVE_GETMNTINFO)
 typedef struct statfs MountTableEntry;
+#define MOUNT_TABLE_ENTRY_TYPE(ent) ((ent)->f_fstypename)
 #else
 typedef struct mntent MountTableEntry;
+#define MOUNT_TABLE_ENTRY_TYPE(ent) ((ent)->mnt_type)
 #endif
 
 typedef struct {
@@ -210,7 +213,6 @@ static GList *         finish_creating_volume_and_prepend       (NautilusVolumeM
 								 NautilusVolume             *volume,
 								 const char                 *file_system_type_name,
 								 GList                      *volume_list);
-static NautilusVolume *copy_volume                              (const NautilusVolume       *volume);
 static void            find_volumes                             (NautilusVolumeMonitor      *monitor);
 static void            free_mount_list                          (GList                      *mount_list);
 static GList *         copy_mount_list                          (GList                      *mount_list);
@@ -218,8 +220,9 @@ static GList *         get_removable_volumes                    (NautilusVolumeM
 static GHashTable *    create_readable_mount_point_name_table   (void);
 static int             get_cdrom_type                           (const char                 *vol_dev_path,
 								 int                        *fd);
-static void            nautilus_volume_free                     (NautilusVolume             *volume);
 static void            nautilus_file_system_type_free           (NautilusFileSystemType     *type);
+static gboolean        entry_is_supermounted_volume             (const MountTableEntry *ent, 
+								 const NautilusVolume *volume);
 
 #ifdef HAVE_CDDA
 static gboolean        locate_audio_cd                          (void);
@@ -493,7 +496,8 @@ has_removable_mntent_options (MountTableEntry *ent)
 	/* Use "owner" or "user" or "users" as our way of determining a removable volume */
 	if (hasmntopt (ent, "user") != NULL
 	    || hasmntopt (ent, "users") != NULL
-	    || hasmntopt (ent, "owner") != NULL) {
+	    || hasmntopt (ent, "owner") != NULL
+	    || eel_strcmp("supermount", MOUNT_TABLE_ENTRY_TYPE (ent)) == 0) {
 		return TRUE;
 	}
 #endif
@@ -522,25 +526,27 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 	GList *volumes;
 	MountTableEntry *ent;
 	NautilusVolume *volume;
+	char * fs_opt;
 #ifdef HAVE_SYS_MNTTAB_H
         MountTableEntry ent_storage;
+#endif
+#ifdef HAVE_GETMNTINFO
+	int count, index;
 #endif
 	ent = NULL;
 	volume = NULL;
 	volumes = NULL;
 
 #ifdef HAVE_GETMNTINFO
-	int count, index;
-	
 	count = getmntinfo (&ent, MNT_WAIT);
 	/* getmentinfo returns a pointer to static data. Do not free. */
 	for (index = 0; index < count; index++) {
-		if (has_removable_mntent_options (ent + 1)) {
+		if (has_removable_mntent_options (&ent[index])) {
 			volume = create_volume (ent[index].f_mntfromname,
-						ent[index].f_mntoname);
+						ent[index].f_mntonname);
 			volume->is_removable = TRUE;
 			volumes = finish_creating_volume_and_prepend
-				(monitor, volume, ent[index].f_fstyename, volumes);
+				(monitor, volume, ent[index].f_fstypename, volumes);
 		}
 	}
 #endif
@@ -563,7 +569,20 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 #elif defined (HAVE_MNTENT_H)
 	while ((ent = getmntent (file)) != NULL) {
 		if (has_removable_mntent_options (ent)) {
-			volume = create_volume (ent->mnt_fsname, ent->mnt_dir);
+#if defined (HAVE_HASMNTOPT)
+
+			if (eel_strcmp("supermount", ent->mnt_type) == 0) {
+				fs_opt = eel_str_strip_substring_and_after (hasmntopt (ent, "dev="),
+									    ",");
+				volume = create_volume (fs_opt+strlen("dev="), ent->mnt_dir);
+				g_free (fs_opt);
+
+			} else {
+#endif
+				volume = create_volume (ent->mnt_fsname, ent->mnt_dir);
+#if defined (HAVE_HASMNTOPT)
+			}
+#endif
 			volumes = finish_creating_volume_and_prepend
 				(monitor, volume, ent->mnt_type, volumes);
 		}
@@ -583,6 +602,23 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 	return g_list_sort (g_list_reverse (volumes), (GCompareFunc) floppy_sort);
 }
 
+static gboolean
+entry_is_supermounted_volume (const MountTableEntry *ent, const NautilusVolume *volume)
+{
+      gboolean result = FALSE;
+#ifdef HAVE_HASMNTOPT
+      char * fs_opt;
+
+      if (strcmp (MOUNT_TABLE_ENTRY_TYPE (ent), "supermount") == 0) {
+              fs_opt = eel_str_strip_substring_and_after (hasmntopt (ent, "dev="),
+                                                          ",");
+              result = strcmp (volume->device_path, fs_opt + strlen ("dev=")) == 0;
+              g_free (fs_opt);
+      }
+#endif
+      return result;
+}
+
 #ifndef SOLARIS_MNT
 
 static gboolean
@@ -617,7 +653,9 @@ volume_is_removable (const NautilusVolume *volume)
 	}
 #elif defined (HAVE_MNTENT_H)
 	while ((ent = getmntent (file)) != NULL) {
-		if (strcmp (volume->device_path, ent->mnt_fsname) == 0
+		if ((strcmp (volume->device_path, ent->mnt_fsname) == 0
+		     || entry_is_supermounted_volume (ent, volume))
+		    && strcmp (volume->mount_path, ent->mnt_dir) == 0
 		    && has_removable_mntent_options (ent)) {
 			removable = TRUE;
 			break;
@@ -631,57 +669,6 @@ volume_is_removable (const NautilusVolume *volume)
 
 #endif /* !SOLARIS_MNT */
 
-static gboolean 
-volume_is_automounted (const NautilusVolume *volume)
-{
-	FILE *file;
-	MountTableEntry *ent;
-	gboolean automounted;
-#ifdef HAVE_SYS_MNTTAB_H
- 	MountTableEntry ent_storage;
-#endif
-	
-	automounted = FALSE; 
- 
-#ifdef HAVE_SYS_MNTTAB_H
-	file = setmntent (MOUNT_TABLE_PATH, "r");
- 	if (file == NULL) {
- 		return FALSE;
- 	}
-
- 	 /* Search for our device in the fstab */
- 	ent = &ent_storage;
-	while (!getmntent (file, ent)) {
-		if ((strstr (ent->mnt_fstype, "autofs") != NULL) 
-		    && (strncmp (ent->mnt_mountp, volume->mount_path, strlen (ent->mnt_mountp)) == 0)) {
-			automounted = TRUE;
-			break;
-		}
- 	}
-	fclose (file);
-
-#elif defined (HAVE_MNTENT_H)
-	file = setmntent (_PATH_MOUNTED, "r");
-	if (file == NULL) {
-		return FALSE;
-	}
-		
-	/* Search for our device in the fstab */
-	while ((ent = getmntent (file)) != NULL) {
-		if ((strstr (ent->mnt_type, "autofs") != NULL) 
-		    && (strncmp (ent->mnt_dir, volume->mount_path, strlen (ent->mnt_dir)) == 0)) {
-			automounted = TRUE;
-			break;
-		}
-	}
-	
-	endmntent (file);
-#endif
-	
-	
-	return automounted;
-}
-
 char *
 nautilus_volume_get_name (const NautilusVolume *volume)
 {
@@ -779,6 +766,26 @@ nautilus_volume_get_device_type (const NautilusVolume *volume)
 	return volume->device_type;
 }
 
+gboolean
+nautilus_volume_is_equal (const NautilusVolume       *volume1,
+			  const NautilusVolume       *volume2)
+{
+	if (strcmp (volume1->mount_path, volume2->mount_path) != 0) {
+		return FALSE;
+	}
+	if (strcmp (volume1->device_path, volume2->device_path) != 0) {
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
+guint
+nautilus_volume_hash (const NautilusVolume       *volume)
+{
+	return g_str_hash (volume->mount_path);
+}
+
 
 /* create_readable_mount_point_name_table
  *
@@ -823,7 +830,7 @@ mount_volume_make_cdrom_name (NautilusVolume *volume)
 		break;
 
 	default:
-		name = NULL;
+		name = g_strdup (_("CD-ROM"));
   	}
 	
 	close (fd);
@@ -872,44 +879,21 @@ mount_volume_activate (NautilusVolumeMonitor *monitor, NautilusVolume *volume)
 }
 
 
-static void *
-eject_device (void *arg)
+static void
+eject_device (const char *path)
 {
-	char *command, *path;	
+	char *command;	
 	
-	path = arg;
-
-	if (path != NULL) {		
+	if (path != NULL) {
 		command = g_strdup_printf ("eject %s", path);	
 		eel_gnome_shell_execute (command);
 		g_free (command);
-		g_free (path);
 	}
-	
-	pthread_exit (NULL);
-
-	/* compilation on Solaris warns of no return 
-	   value on non-void function...so....*/
-	return (void *) 0;
 }
 
 static void
 mount_volume_deactivate (NautilusVolumeMonitor *monitor, NautilusVolume *volume)
 {
-	pthread_t eject_thread;
-
-	switch (volume->device_type) {
-	case NAUTILUS_DEVICE_CDROM_DRIVE:
-		/* Don't eject automounted device 
-		 * since this action is done by automount daemon, not user */
-		if (!volume_is_automounted (volume)) {
-			pthread_create (&eject_thread, NULL, eject_device, g_strdup (volume->device_path));
-		}
-		break;
-	default:
-		break;
-	}
-
 	g_signal_emit (monitor,
 			 signals[VOLUME_UNMOUNTED], 0,
 			 volume);
@@ -932,7 +916,7 @@ copy_mount_list (GList *mount_list)
 	while (list) {
 		volume = list->data;
 
-		new_list = g_list_prepend (new_list, copy_volume (volume));
+		new_list = g_list_prepend (new_list, nautilus_volume_copy (volume));
 				
 		list = list->next;
 	}
@@ -969,7 +953,7 @@ build_volume_list_delta (GList *list_one, GList *list_two)
 		
 		if (!found_match) {
 			/* No match. Add it to the list to be returned; */
-			new_volume = copy_volume (volOne);
+			new_volume = nautilus_volume_copy (volOne);
 			new_list = g_list_prepend (new_list, new_volume);
 		}
 	}
@@ -1040,6 +1024,7 @@ get_mount_list (NautilusVolumeMonitor *monitor)
 	static time_t last_mtime = 0;
         static FILE *fh = NULL;
         static GList *saved_list = NULL;
+	GList * removable_list;				
         const char *file_name;
 	const char *separator;
 	char line[PATH_MAX * 3];
@@ -1100,6 +1085,19 @@ get_mount_list (NautilusVolumeMonitor *monitor)
                         device_path = eel_string_list_nth (list, 0);
                         mount_path = eel_string_list_nth (list, 1);
                         file_system_type_name = eel_string_list_nth (list, 2);
+ 			/* For supermount, search info in removable list */
+ 			if (eel_strcmp ("supermount", file_system_type_name) == 0) {
+ 				removable_list = monitor->details->removable_volumes;
+ 				while (removable_list) {
+ 					volume = (NautilusVolume *) removable_list->data;
+ 					if (eel_strcmp (mount_path, volume->mount_path) == 0) {
+						g_free (device_path);
+ 						device_path = g_strdup (volume->device_path);
+ 						break;
+ 					}
+ 					removable_list = removable_list->next;
+ 				}				
+ 			}
                         volume = create_volume (device_path, mount_path);
 			if (eel_string_list_get_length (list) >= 4 &&
 			    option_list_has_option (eel_string_list_peek_nth (list, 3), MNTOPT_RO))
@@ -1425,7 +1423,9 @@ open_error_pipe (void)
 typedef struct {
 	char *command;
 	char *mount_point;
+	char *device_path;
 	gboolean should_mount;
+	gboolean should_eject;
 } MountThreadInfo;
 
 typedef struct {
@@ -1572,10 +1572,17 @@ mount_unmount_callback (void *arg)
 		old_locale = g_getenv ("LC_ALL");
 		eel_setenv ("LC_ALL", "C", TRUE);
 
-		open_error_pipe ();
-		file = popen (info->command, "r");
-		close_error_pipe (info->should_mount, info->mount_point);
-		pclose (file);
+		if (info->command != NULL) {
+			open_error_pipe ();
+			file = popen (info->command, "r");
+			close_error_pipe (info->should_mount, info->mount_point);
+			pclose (file);
+			g_free (info->command);
+		}
+
+		if (info->should_eject) {
+			eject_device (info->device_path?info->device_path:info->mount_point);
+		}
 		
 		if (old_locale != NULL) {
 			eel_setenv ("LC_ALL", old_locale, TRUE);
@@ -1583,8 +1590,8 @@ mount_unmount_callback (void *arg)
 			eel_unsetenv("LC_ALL");
 		}
 
-		g_free (info->command);
 		g_free (info->mount_point);
+		g_free (info->device_path);
 		g_free (info);
 	}
 	
@@ -1597,7 +1604,8 @@ mount_unmount_callback (void *arg)
 void
 nautilus_volume_monitor_mount_unmount_removable (NautilusVolumeMonitor *monitor,
 						 const char *mount_point,
-						 gboolean should_mount)
+						 gboolean should_mount,
+						 gboolean should_eject)
 {
 	const char *command;
 	GList *p;
@@ -1644,9 +1652,17 @@ nautilus_volume_monitor_mount_unmount_removable (NautilusVolumeMonitor *monitor,
        }
 
 	mount_info = g_new0 (MountThreadInfo, 1);
-	mount_info->command = g_strdup (command_string);	
+       /* Don't run mount/umount on supermount mount point, it is useless */
+        if ((volume->file_system_type == NULL) 
+	    || (strcmp (volume->file_system_type->name,"supermount") != 0)) {
+		mount_info->command = g_strdup (command_string);	
+	}
 	mount_info->mount_point = g_strdup (mount_point);
+	if (volume) {
+		mount_info->device_path = g_strdup (volume->device_path);
+	}
 	mount_info->should_mount = should_mount;
+	mount_info->should_eject = should_eject;
 	
 	pthread_create (&mount_thread, NULL, mount_unmount_callback, mount_info);
 	
@@ -1686,8 +1702,8 @@ create_volume (const char *device_path, const char *mount_path)
 	return volume;
 }
 
-static NautilusVolume *
-copy_volume (const NautilusVolume *volume)
+NautilusVolume *
+nautilus_volume_copy (const NautilusVolume *volume)
 {
 	NautilusVolume *new_volume;
 
@@ -1766,7 +1782,11 @@ finish_creating_volume (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
 	gboolean ok;
 	const char *name;
 	struct stat statbuf;
-	
+
+	if (stat (volume->mount_path, &statbuf) == 0) {
+		volume->device = statbuf.st_dev;
+	}
+
 	volume->file_system_type = g_hash_table_lookup
 		(monitor->details->file_system_table, file_system_type_name);
 
@@ -1802,10 +1822,6 @@ finish_creating_volume (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
 		return FALSE;
 	}
 
-	if (stat (volume->mount_path, &statbuf) == 0) {
-		volume->device = statbuf.st_dev;
-	}
-
 	/* Identify device type */
 	if (eel_str_has_prefix (volume->mount_path, "/mnt/")) {		
 		name = volume->mount_path + strlen ("/mnt/");
@@ -1851,9 +1867,13 @@ finish_creating_volume (NautilusVolumeMonitor *monitor, NautilusVolume *volume,
 			} else if (eel_str_has_prefix (volume->device_path, floppy_device_path_prefix)) {
 				volume->device_type = NAUTILUS_DEVICE_FLOPPY_DRIVE;
 				volume->is_removable = TRUE;
-			} else if (eel_str_has_prefix (name, "zip")) {
+			} else if (eel_str_has_prefix (name, "rmdisk")) {
 				volume->device_type = NAUTILUS_DEVICE_ZIP_DRIVE;
 				volume->is_removable = TRUE;
+				if( eel_str_has_suffix (volume->device_path, ":c")) {
+					volume->device_path = eel_str_strip_trailing_str
+								(volume->device_path, ":c");
+				}
 			} else if (eel_str_has_prefix (name, "jaz")) {
 				volume->device_type = NAUTILUS_DEVICE_JAZ_DRIVE;
 				volume->is_removable = TRUE;
